@@ -7,25 +7,25 @@ logger = logging.getLogger('tauk')
 
 
 @unique
-class TestStatus(Enum):
-    FAILED = 'failed'
-    PASSED = 'passed'
-    EXCLUDED = 'excluded'
-
+class TaukEnum(Enum):
     def __getstate__(self):
         return self.value
 
 
 @unique
-class BrowserNames(Enum):
+class TestStatus(TaukEnum):
+    FAILED = 'failed'
+    PASSED = 'passed'
+    EXCLUDED = 'excluded'
+
+
+@unique
+class BrowserNames(TaukEnum):
     CHROME = 'chrome'
     FIREFOX = 'firefox'
     EDGE = 'edge'
     SAFARI = 'safari'
 
-    def __getstate__(self):
-        return self.value
-
     @classmethod
     def resolve(cls, name: str):
         if 'chrome' in name.lower():
@@ -41,27 +41,21 @@ class BrowserNames(Enum):
 
 
 @unique
-class AutomationTypes(Enum):
+class AutomationTypes(TaukEnum):
     APPIUM = 'appium'
     SELENIUM = 'selenium'
     ESPRESSO = 'espresso'
     XCTEST = 'xctest'
 
-    def __getstate__(self):
-        return self.value
-
 
 @unique
-class PlatformNames(Enum):
+class PlatformNames(TaukEnum):
     IOS = 'ios'
     ANDROID = 'android'
     WINDOWS = 'windows'
     LINUX = 'linux'
     MACOS = 'macos'
 
-    def __getstate__(self):
-        return self.value
-
     @classmethod
     def resolve(cls, name: str):
         if 'mac' in name.lower():
@@ -76,3 +70,30 @@ class PlatformNames(Enum):
             return PlatformNames.IOS
         else:
             raise TaukException(f'unable to resolve platform name {name}')
+
+
+@unique
+class AttachmentTypes(TaukEnum):
+    ASSISTANT_CONSOLE_LOGS = 'Runtime.consoleLogs'
+    ASSISTANT_EXCEPTION_LOGS = 'Runtime.exceptionLogs'
+    ASSISTANT_BROWSER_LOGS = 'Log.browserLogs'
+
+    @classmethod
+    def resolve_assistant_log(cls, name: str):
+        if 'console_logs.json' in name.lower():
+            return AttachmentTypes.ASSISTANT_CONSOLE_LOGS
+        elif 'exception_logs.json' in name.lower():
+            return AttachmentTypes.ASSISTANT_EXCEPTION_LOGS
+        elif 'browser_logs.json' in name.lower():
+            return AttachmentTypes.ASSISTANT_BROWSER_LOGS
+        else:
+            raise TaukException(f'unable to resolve platform name {name}')
+
+    @classmethod
+    def is_assistant_attachment(cls, attachment_type):
+        if attachment_type in [AttachmentTypes.ASSISTANT_EXCEPTION_LOGS,
+                               AttachmentTypes.ASSISTANT_EXCEPTION_LOGS,
+                               AttachmentTypes.ASSISTANT_BROWSER_LOGS]:
+            return True
+
+        return False
